package com.innosoft.ctgen.common.converter;

import com.itextpdf.text.*;
import com.itextpdf.text.Image;
import com.itextpdf.text.pdf.PdfWriter;
import com.spire.xls.FileFormat;
import com.spire.xls.Workbook;
import org.apache.commons.compress.utils.Lists;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.apache.poi.hslf.usermodel.HSLFSlide;
import org.apache.poi.hslf.usermodel.HSLFSlideShow;
import org.apache.poi.sl.usermodel.Sheet;
import org.apache.poi.xslf.usermodel.XMLSlideShow;
import org.apache.poi.xslf.usermodel.XSLFSlide;

import java.awt.Color;
import java.io.FileInputStream;
import java.io.*;

import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.util.List;

/**
 * 파워포인트(ppt, pptx) 파일을 PDF로 converting한다.
 */
public class OfficeToPDFConverter {

    static Logger logger = LogManager.getLogger();

    String realFolder = "FileManager.realFolder";
    private String pdfFileName;

    private String pdfFilePath;


    public OfficeToPDFConverter(){
    }

    public OfficeToPDFConverter(String filePath, String fileName){
        try {
            this.realFolder = filePath;
            setPdfFileName(this.convert(fileName));
        } catch (IOException | DocumentException e) {
            e.printStackTrace();
        }
    }

    /**
     * 컨버트한 PDF파일 이름을 set한다.
     * @param pdfFileName
     */
    private void setPdfFileName(String pdfFileName) {
        this.pdfFileName = pdfFileName;
    }

    /**
     * 컨버트한 PDF파일 이름을 get한다.
     * @return
     */
    public String getPdfFileName() {
        return pdfFileName;
    }

    private String convert(String fileName) throws IOException, DocumentException{


        String pdfFileName = "";
        if (fileName.endsWith(".pptx")){
            pdfFileName = createPDF(convertPPTXtoPDF(fileName), fileName);
        }
        else if (fileName.endsWith(".ppt")){
            pdfFileName = createPDF(convertPPTtoPDF(fileName), fileName);
        }
        else if (fileName.endsWith(".xls") || fileName.endsWith(".xlsx")){
            pdfFileName = convertXLStoPDF(fileName);
        }

        return pdfFileName;
    }

    /**
     * 이미지 파일을 PDF로 저장한다.
     * @param imgList
     * @param fileName
     * @return
     * @throws IOException
     * @throws DocumentException
     */
    private String createPDF(List<String> imgList, String fileName) throws IOException, DocumentException {
        String[] filenameArr = fileName.split("\\.");
        Document document = new Document(PageSize.A4.rotate(), 35,0,35,0);
        File dir = new File(realFolder);
        if (!dir.isDirectory()) {
            dir.mkdirs();
        }

        String pdfFileName = filenameArr[0] + ".pdf";
        PdfWriter.getInstance(document, new FileOutputStream(realFolder + pdfFileName));
        document.open();

        for(int i=0; i < imgList.size(); i++){
            String imagePath = imgList.get(i);
            Image image = Image.getInstance(imagePath);
            document.add(image);

            //4.Delete Image

        }

        document.close();

        return pdfFileName;
    }

    /**
     * PowerPoint 2007 이전 ppt 파일을 이미지로 저장한다.
     * @param fileName
     * @return
     * @throws IOException
     */
    private List<String> convertPPTtoPDF(String fileName) throws IOException {
        String[] fileNameArr = fileName.split("\\.");
        List<String> imagePathList = Lists.newArrayList();

        FileInputStream fis = new FileInputStream(realFolder + fileName);
        HSLFSlideShow ppt = new HSLFSlideShow(fis);
        fis.close();

        Dimension dimension = ppt.getPageSize();
        List<HSLFSlide> slideList = ppt.getSlides();
        int index = 0;
        for (HSLFSlide slide : slideList) {

            File imageFile = new File(realFolder + fileNameArr[0] + (index) + ".png");
            convertSlideToImage(slide, dimension, imageFile);
            imagePathList.add(imageFile.getAbsolutePath());

            index++;
        }

        return imagePathList;
    }

    private String convertXLStoPDF(String fileName) throws IOException, DocumentException {

        String[] fileNameArr = fileName.split("\\.");
        Workbook wb = new Workbook();
        wb.loadFromFile(realFolder + fileName);

        wb.saveToFile(realFolder + fileNameArr[0] + ".pdf", FileFormat.PDF);

        return fileNameArr[0] + ".pdf";
    }

    /**
     * PowerPoint 2007 이후 버전은 pptx 파일을 이미지로 저장한다.
     * @param fileName
     * @return
     * @throws IOException
     */
    private List<String> convertPPTXtoPDF(String fileName) throws IOException {
        List<String> imagePathList = Lists.newArrayList();

        String[] fileNameArr = fileName.split("\\.");
//        String pptFilePath = realFolder;
        FileInputStream fis = new FileInputStream(realFolder + fileName);
        XMLSlideShow ppt = new XMLSlideShow(fis);
        fis.close();

        Dimension dimension = ppt.getPageSize();
        List<XSLFSlide> slideList = ppt.getSlides();
        int index = 0;
        for (XSLFSlide slide : slideList) {

            File imageFile = new File(realFolder + fileNameArr[0] + (index) + ".png");
            convertSlideToImage(slide, dimension, imageFile);
            imagePathList.add(imageFile.getAbsolutePath());

            index++;
        }

        return imagePathList;
    }

    /**
     * 파워포인트 슬라이드를 이미지로 Converter
     * @param sheet
     * @param dimension
     * @param imageFile
     */
    private static void convertSlideToImage(Sheet sheet, Dimension dimension, File imageFile) {
        BufferedImage img = new BufferedImage(dimension.width, dimension.height, BufferedImage.TYPE_INT_ARGB);
        Graphics2D graphics = img.createGraphics();

        graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                RenderingHints.VALUE_ANTIALIAS_ON);
        graphics.setRenderingHint(RenderingHints.KEY_RENDERING,
                RenderingHints.VALUE_RENDER_QUALITY);
        graphics.setRenderingHint(RenderingHints.KEY_INTERPOLATION,
                RenderingHints.VALUE_INTERPOLATION_BICUBIC);
        graphics.setRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS,
                RenderingHints.VALUE_FRACTIONALMETRICS_ON);

        graphics.setColor(Color.white);
        graphics.fill(new Rectangle2D.Float(0, 0, dimension.width, dimension.height));

        sheet.draw(graphics);

        FileOutputStream out = null;
        try {
            out = new FileOutputStream(imageFile);
            ImageIO.write(img, "png", out);
        } catch (FileNotFoundException e) {
            logger.error("FileNotFoundException", e);
            e.printStackTrace();
        } catch (IOException e) {
            logger.error("IOException", e);
            e.printStackTrace();
        } finally {
            try {
                out.close();
            } catch (IOException e) {
                logger.error("IOException FileOutputStream Close Fail", e);
                e.printStackTrace();
                e.printStackTrace();
            }
        }

    }
}